﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Color_Extensions;

namespace GBAFE_Tilesets
{
    public partial class ColorSelector : UserControl
    {
        private bool updating;
        private Color_Bit_Depth Bit_Depth, _DefaultBitDepth = Color_Bit_Depth.None;
        private Color Active_Color;

        internal Color active_color
        {
            get { return Active_Color; }
            set
            {
                Active_Color = YetiGraphics.convert_color_bit_depth(value, Bit_Depth, Color_Bit_Depth.R8G8B8);
                refresh_spinners();
                if (ColorChanged != null)
                    ColorChanged(this, new EventArgs());
            }
        }
        [Browsable(true), DefaultValue(Color_Bit_Depth.None)]
        public Color_Bit_Depth DefaultBitDepth
        {
            get { return _DefaultBitDepth; }
            set
            {
                _DefaultBitDepth = value;
                if (value != Color_Bit_Depth.None)
                {
                    update_color_depth(value, Bit_Depth);
                    ColorDepthComboBox.Enabled = false;
                    ColorDepthComboBox.Visible = false;
                }
                else
                {
                    ColorDepthComboBox.Enabled = true;
                    ColorDepthComboBox.Visible = true;
                }
            }
        }

        public ColorSelector()
        {
            InitializeComponent();
            Bit_Depth = Color_Bit_Depth.Red8 | Color_Bit_Depth.Green8 | Color_Bit_Depth.Blue8;

            refresh_spinners();
            ColorDepthComboBox.SelectedIndex = 0;

            this.RedTrackBar.Scroll += new EventHandler(this.RedTrackBar_Scroll);
            this.GreenTrackBar.Scroll += new EventHandler(this.GreenTrackBar_Scroll);
            this.BlueTrackBar.Scroll += new EventHandler(this.BlueTrackBar_Scroll);
            this.LightnessHueTrackBar.Scroll += new EventHandler(this.LightnessHueTrackBar_Scroll);
            this.LightnessSaturationTrackBar.Scroll += new EventHandler(this.LightnessSaturationTrackBar_Scroll);
            this.LightnessTrackBar.Scroll += new EventHandler(this.LightnessTrackBar_Scroll);
            this.ValueHueTrackBar.Scroll += new EventHandler(this.ValueHueTrackBar_Scroll);
            this.ValueSaturationTrackBar.Scroll += new EventHandler(this.ValueSaturationTrackBar_Scroll);
            this.ValueTrackBar.Scroll += new EventHandler(this.ValueTrackBar_Scroll);
        }

        private void refresh_spinners()
        {
            updating = true;
            refresh_rgb_spinners();
            refresh_hsl_spinners();
            refresh_hsv_spinners();
            refresh_trackbar_color();
            updating = false;
        }
        private void refresh_rgb_spinners()
        {
            RedSpinner.Value = (active_color.R * YetiGraphics.channel_bit_depth(Bit_Depth, Color_Channels.Red)) / 256;
            GreenSpinner.Value = (active_color.G * YetiGraphics.channel_bit_depth(Bit_Depth, Color_Channels.Green)) / 256;
            BlueSpinner.Value = (active_color.B * YetiGraphics.channel_bit_depth(Bit_Depth, Color_Channels.Blue)) / 256;
        }
        private void refresh_hsl_spinners()
        {
            LightnessHueSpinner.Value = (int)active_color.GetHue();
            LightnessSaturationSpinner.Value = (int)(active_color.GetSaturation() * (double)LightnessSaturationSpinner.Maximum);
            LightnessSpinner.Value = (int)(active_color.GetBrightness() * (double)LightnessSpinner.Maximum);
        }
        private void refresh_hsv_spinners()
        {
            ValueHueSpinner.Value = (int)active_color.GetHue();
            ValueSaturationSpinner.Value = (int)(active_color.GetHSLSaturation() * (double)ValueSaturationSpinner.Maximum);
            ValueSpinner.Value = (int)(active_color.GetValue() * (double)ValueSpinner.Maximum);
        }
        private void refresh_trackbar_color()
        {
            RedTrackBar.color = active_color;
            GreenTrackBar.color = active_color;
            BlueTrackBar.color = active_color;

            LightnessHueTrackBar.Value = (int)LightnessHueSpinner.Value;
            LightnessHueTrackBar.color = active_color;
            LightnessSaturationTrackBar.Value = (int)LightnessSaturationSpinner.Value;
            LightnessSaturationTrackBar.color = YetiGraphics.color_from_hsl((int)LightnessHueSpinner.Value, 1, 0.5, Bit_Depth);
            LightnessTrackBar.Value = (int)LightnessSpinner.Value;
            LightnessTrackBar.color = YetiGraphics.color_from_hsl((int)LightnessHueSpinner.Value,
                ((double)LightnessSaturationSpinner.Value / (double)LightnessSaturationSpinner.Maximum), 0.5, Bit_Depth);

            ValueHueTrackBar.Value = (int)ValueHueSpinner.Value;
            ValueHueTrackBar.color = active_color;
            ValueSaturationTrackBar.Value = (int)ValueSaturationSpinner.Value;
            ValueSaturationTrackBar.color = active_color;
            ValueTrackBar.Value = (int)ValueSpinner.Value;
            ValueTrackBar.color = YetiGraphics.color_from_hsv((int)ValueHueSpinner.Value,
                ((double)ValueSaturationSpinner.Value / (double)ValueSaturationSpinner.Maximum), 1f, Bit_Depth);

            RedTrackBar.Invalidate();
            GreenTrackBar.Invalidate();
            BlueTrackBar.Invalidate();
            LightnessHueTrackBar.Invalidate();
            LightnessSaturationTrackBar.Invalidate();
            LightnessTrackBar.Invalidate();
            ValueHueTrackBar.Invalidate();
            ValueSaturationTrackBar.Invalidate();
            ValueTrackBar.Invalidate();
        }
        private void refresh_rgb()
        {
            int r = ((int)RedSpinner.Value) * (256 / YetiGraphics.channel_bit_depth(Bit_Depth, Color_Channels.Red));
            int g = ((int)GreenSpinner.Value) * (256 / YetiGraphics.channel_bit_depth(Bit_Depth, Color_Channels.Green));
            int b = ((int)BlueSpinner.Value) * (256 / YetiGraphics.channel_bit_depth(Bit_Depth, Color_Channels.Blue));
            Color color = Color.FromArgb(255, r, g, b);
            Active_Color = color;

            refresh_hsl_spinners();
            refresh_hsv_spinners();
            refresh_trackbar_color();

            if (ColorChanged != null)
                ColorChanged(this, new EventArgs());
            ColorEdited(this, new EventArgs());
        }
        private void refresh_hsl()
        {
            Color color = YetiGraphics.color_from_hsl((double)LightnessHueSpinner.Value * (360 / (double)LightnessHueSpinner.Maximum),
                (double)LightnessSaturationSpinner.Value / (double)LightnessSaturationSpinner.Maximum,
                (double)LightnessSpinner.Value / (double)LightnessSpinner.Maximum, Bit_Depth);
            Active_Color = color;

            refresh_rgb_spinners();
            refresh_hsv_spinners();
            ValueHueSpinner.Value = LightnessHueSpinner.Value;
            refresh_trackbar_color();

            if (ColorChanged != null)
                ColorChanged(this, new EventArgs());
            ColorEdited(this, new EventArgs());
        }
        private void refresh_hsv()
        {
            Color color = YetiGraphics.color_from_hsv((double)ValueHueSpinner.Value * (360 / (double)ValueHueSpinner.Maximum),
                (double)ValueSaturationSpinner.Value / (double)ValueSaturationSpinner.Maximum,
                (double)ValueSpinner.Value / (double)ValueSpinner.Maximum, Bit_Depth);
            Active_Color = color;

            refresh_rgb_spinners();
            refresh_hsl_spinners();
            LightnessHueSpinner.Value = ValueHueSpinner.Value;
            refresh_trackbar_color();

            if (ColorChanged != null)
                ColorChanged(this, new EventArgs());
            ColorEdited(this, new EventArgs());
        }

        private void update_color_depth(Color_Bit_Depth new_bit_depth, Color_Bit_Depth old_bit_depth)
        {
            updating = true;
            Bit_Depth = new_bit_depth;

            if (BitDepthChanged != null)
                BitDepthChanged(this, new BitDepthChangeEventArgs(new_bit_depth, old_bit_depth));

            RedSpinner.Maximum = RedTrackBar.Maximum = YetiGraphics.channel_bit_depth(Bit_Depth, Color_Channels.Red) - 1;
            GreenSpinner.Maximum = GreenTrackBar.Maximum = YetiGraphics.channel_bit_depth(Bit_Depth, Color_Channels.Green) - 1;
            BlueSpinner.Maximum = BlueTrackBar.Maximum = YetiGraphics.channel_bit_depth(Bit_Depth, Color_Channels.Blue) - 1;

            refresh_rgb_spinners();
            refresh_hsl_spinners();
            refresh_hsv_spinners();
            refresh_trackbar_color();

            updating = false;
        }

        internal delegate void BitDepthChangeEventHandler(object sender, BitDepthChangeEventArgs e);
        internal event BitDepthChangeEventHandler BitDepthChanged;
        /// <summary>
        /// Occurs when the color changed for any reason; for use in updating swatches.
        /// </summary>
        internal event EventHandler ColorChanged;
        /// <summary>
        /// Occurs when the color is manually edited by the user.
        /// </summary>
        internal event EventHandler ColorEdited;

        #region Events
        private void RedSpinner_ValueChanged(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                refresh_rgb();
                updating = false;
            }
        }
        private void GreenSpinner_ValueChanged(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                refresh_rgb();
                updating = false;
            }
        }
        private void BlueSpinner_ValueChanged(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                refresh_rgb();
                updating = false;
            }
        }

        private void RedTrackBar_Scroll(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                RedSpinner.Value = RedTrackBar.Value;
                refresh_rgb();
                updating = false;
            }
        }
        private void GreenTrackBar_Scroll(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                GreenSpinner.Value = GreenTrackBar.Value;
                refresh_rgb();
                updating = false;
            }
        }
        private void BlueTrackBar_Scroll(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                BlueSpinner.Value = BlueTrackBar.Value;
                refresh_rgb();
                updating = false;
            }
        }

        private void LightnessHueSpinner_ValueChanged(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                refresh_hsl();
                updating = false;
            }
        }
        private void LightnessSaturationSpinner_ValueChanged(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                refresh_hsl();
                updating = false;
            }
        }
        private void LightnessSpinner_ValueChanged(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                refresh_hsl();
                updating = false;
            }
        }

        private void LightnessHueTrackBar_Scroll(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                LightnessHueSpinner.Value = LightnessHueTrackBar.Value;
                refresh_hsl();
                updating = false;
            }
        }
        private void LightnessSaturationTrackBar_Scroll(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                LightnessSaturationSpinner.Value = LightnessSaturationTrackBar.Value;
                refresh_hsl();
                updating = false;
            }
        }
        private void LightnessTrackBar_Scroll(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                LightnessSpinner.Value = LightnessTrackBar.Value;
                refresh_hsl();
                updating = false;
            }
        }

        private void ValueHueSpinner_ValueChanged(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                refresh_hsv();
                updating = false;
            }
        }
        private void ValueSaturationSpinner_ValueChanged(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                refresh_hsv();
                updating = false;
            }
        }
        private void ValueSpinner_ValueChanged(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                refresh_hsv();
                updating = false;
            }
        }

        private void ValueHueTrackBar_Scroll(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                ValueHueSpinner.Value = ValueHueTrackBar.Value;
                refresh_hsv();
                updating = false;
            }
        }
        private void ValueSaturationTrackBar_Scroll(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                ValueSaturationSpinner.Value = ValueSaturationTrackBar.Value;
                refresh_hsv();
                updating = false;
            }
        }
        private void ValueTrackBar_Scroll(object sender, EventArgs e)
        {
            if (!updating)
            {
                updating = true;
                ValueSpinner.Value = ValueTrackBar.Value;
                refresh_hsv();
                updating = false;
            }
        }

        private void ColorDepthComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (ColorDepthComboBox.SelectedIndex)
            {
                case 0:
                    update_color_depth(Color_Bit_Depth.R8G8B8, Bit_Depth);
                    break;
                case 1:
                    update_color_depth(Color_Bit_Depth.R5G5B5, Bit_Depth);
                    break;
                case 2:
                    update_color_depth(Color_Bit_Depth.R5G6B5, Bit_Depth);
                    break;
            }
        }
        #endregion
    }
}
