﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;
using Color_Extensions;

namespace GBAFE_Tilesets
{
    public enum Color_Trackbar_Modes { Red, Green, Blue, Hue, LightSat, Lightness, ValueSat, Value }
    class Color_TrackBar : System.Windows.Forms.TrackBar
    {
        private Rectangle trackRectangle = new Rectangle();
        private Rectangle ticksRectangle = new Rectangle();
        private Rectangle thumbRectangle = new Rectangle();
        private bool thumbClicked = false;
        private TrackBarThumbState thumbState = TrackBarThumbState.Normal;

        private Color Active_Color;
        private Color_Trackbar_Modes Color_Mode;

        #region Accessors
        public Color color
        {
            set
            {
                int new_value = Value;
                Active_Color = value;
                switch (Color_Mode)
                {
                    case Color_Trackbar_Modes.Red:
                        new_value = (value.R * (Maximum + 1)) / 256;
                        break;
                    case Color_Trackbar_Modes.Green:
                        new_value = (value.G * (Maximum + 1)) / 256;
                        break;
                    case Color_Trackbar_Modes.Blue:
                        new_value = (value.B * (Maximum + 1)) / 256;
                        break;
                    case Color_Trackbar_Modes.Hue:
                    case Color_Trackbar_Modes.LightSat:
                    case Color_Trackbar_Modes.Lightness:
                    case Color_Trackbar_Modes.ValueSat:
                    case Color_Trackbar_Modes.Value:
                        thumbRectangle.X = CurrentTickXCoordinate();
                        return;
                }
                if (Value != new_value)
                {
                    Value = Math.Min(Maximum, Math.Max(Minimum, new_value));
                    thumbRectangle.X = CurrentTickXCoordinate();
                }
            }
        }
        [DisplayName("Color Mode"), Browsable(true), Description("What color or color component this slider affects")]
        public Color_Trackbar_Modes color_mode
        {
            get { return Color_Mode; }
            set { Color_Mode = value; }
        }
        #endregion

        private Color gradient_start_color
        {
            get
            {
                switch (Color_Mode)
                {
                    case Color_Trackbar_Modes.Red:
                        return Color.FromArgb(0, Active_Color.G, Active_Color.B);
                    case Color_Trackbar_Modes.Green:
                        return Color.FromArgb(Active_Color.R, 0, Active_Color.B);
                    case Color_Trackbar_Modes.Blue:
                        return Color.FromArgb(Active_Color.R, Active_Color.G, 0);
                    case Color_Trackbar_Modes.LightSat:
                        return Color.Gray;
                    case Color_Trackbar_Modes.Lightness:
                    case Color_Trackbar_Modes.Value:
                        return Color.Black;
                    case Color_Trackbar_Modes.ValueSat:
                        return YetiGraphics.color_from_hsv(0, 0, Active_Color.GetBrightness());
                }
                return Color.Black;
            }
        }
        private Color gradient_mid_color
        {
            get
            {
                switch (Color_Mode)
                {
                    case Color_Trackbar_Modes.Lightness:
                        return Active_Color;
                        return YetiGraphics.color_from_hsl(Active_Color.GetHue(), Active_Color.GetSaturation(), 0.5);
                }
                return Color.White;
            }
        }
        private Color gradient_end_color
        {
            get
            {
                switch (Color_Mode)
                {
                    case Color_Trackbar_Modes.Red:
                        return Color.FromArgb(255, Active_Color.G, Active_Color.B);
                    case Color_Trackbar_Modes.Green:
                        return Color.FromArgb(Active_Color.R, 255, Active_Color.B);
                    case Color_Trackbar_Modes.Blue:
                        return Color.FromArgb(Active_Color.R, Active_Color.G, 255);
                    case Color_Trackbar_Modes.LightSat:
                        return YetiGraphics.color_from_hsl(Active_Color.GetHue(), 1, 0.5);
                    case Color_Trackbar_Modes.Lightness:
                        return Color.White;
                    case Color_Trackbar_Modes.ValueSat:
                        return YetiGraphics.color_from_hsv(Active_Color.GetHue(), 1f, Active_Color.GetValue());
                    case Color_Trackbar_Modes.Value:
                        return YetiGraphics.color_from_hsv(Active_Color.GetHue(), Active_Color.GetHSLSaturation(), 1f);
                }
                return Color.White;
            }
        }

        public Color_TrackBar()
        {
            color = Color.Black;
            Color_Mode = Color_Trackbar_Modes.Red;
            if (TrackBarRenderer.IsSupported)
                SetStyle(ControlStyles.UserPaint, true);
            this.DoubleBuffered = true;

            // Calculate the initial sizes of the bar,  
            // thumb and ticks.
            SetupTrackBar();
        }

        // Calculate the sizes of the bar, thumb, and ticks rectangle. 
        private void SetupTrackBar()
        {
            if (!TrackBarRenderer.IsSupported)
                return;

            using (Graphics g = this.CreateGraphics())
            {
                // Calculate the size of the track bar.
                trackRectangle = new Rectangle(ClientRectangle.X + 7, ClientRectangle.Y + 7, ClientRectangle.Width - 14, 4);

                // Calculate the size of the rectangle in which to  
                // draw the ticks.
                ticksRectangle.X = trackRectangle.X + 4;
                ticksRectangle.Y = trackRectangle.Y - 8;
                ticksRectangle.Width = trackRectangle.Width - 8;
                ticksRectangle.Height = 4;

                // Calculate the size of the thumb.
                thumbRectangle.Size =
                    TrackBarRenderer.GetTopPointingThumbSize(g,
                    TrackBarThumbState.Normal);
                thumbRectangle.Size = Size.Add(thumbRectangle.Size, new Size(-2, 0));

                thumbRectangle.X = CurrentTickXCoordinate();
                thumbRectangle.Y = trackRectangle.Y - 7;
            }
        }

        private int CurrentTickXCoordinate()
        {
            if (ticksRectangle.Width == 0)
                return 0;
            return ticksRectangle.X + (int)(ticksRectangle.Width * (Value - Minimum) / (float)((Maximum - Minimum))) - 4;
        }

        protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
        {
            if (!TrackBarRenderer.IsSupported)
            {
                return;
            }

            TrackBarRenderer.DrawHorizontalTrack(e.Graphics, trackRectangle);

            Rectangle gradient_rectangle = new Rectangle(ticksRectangle.X, trackRectangle.Y + 5, ticksRectangle.Width, 8);
            switch (Color_Mode)
            {
                case Color_Trackbar_Modes.Hue:
                    using (Brush brush = new LinearGradientBrush(
                            new PointF(ticksRectangle.X, 0), new PointF(ticksRectangle.X + (ticksRectangle.Width * 1 / 6f), 0),
                            Color.Red, Color.Yellow))
                        e.Graphics.FillRectangle(brush, new RectangleF(
                            gradient_rectangle.X, gradient_rectangle.Y,
                            gradient_rectangle.Width / 6f, gradient_rectangle.Height));
                    using (Brush brush = new LinearGradientBrush(
                            new PointF(ticksRectangle.X + (ticksRectangle.Width * 1 / 6f), 0), new PointF(ticksRectangle.X + (ticksRectangle.Width * 2 / 6f), 0),
                            Color.Yellow, Color.Lime))
                        e.Graphics.FillRectangle(brush, new RectangleF(
                            gradient_rectangle.X + (ticksRectangle.Width * 1 / 6f), gradient_rectangle.Y,
                            gradient_rectangle.Width / 6f, gradient_rectangle.Height));
                    using (Brush brush = new LinearGradientBrush(
                            new PointF(ticksRectangle.X + (ticksRectangle.Width * 2 / 6f), 0), new PointF(ticksRectangle.X + (ticksRectangle.Width * 3 / 6f), 0),
                            Color.Lime, Color.Cyan))
                        e.Graphics.FillRectangle(brush, new RectangleF(
                            gradient_rectangle.X + (ticksRectangle.Width * 2 / 6f), gradient_rectangle.Y,
                            gradient_rectangle.Width / 6f, gradient_rectangle.Height));
                    using (Brush brush = new LinearGradientBrush(
                            new PointF(ticksRectangle.X + (ticksRectangle.Width * 3 / 6f), 0), new PointF(ticksRectangle.X + (ticksRectangle.Width * 4 / 6f), 0),
                            Color.Cyan, Color.Blue))
                        e.Graphics.FillRectangle(brush, new RectangleF(
                            gradient_rectangle.X + (ticksRectangle.Width * 3 / 6f), gradient_rectangle.Y,
                            gradient_rectangle.Width / 6f, gradient_rectangle.Height));
                    using (Brush brush = new LinearGradientBrush(
                            new PointF(ticksRectangle.X + (ticksRectangle.Width * 4 / 6f), 0), new PointF(ticksRectangle.X + (ticksRectangle.Width * 5 / 6f), 0),
                            Color.Blue, Color.Magenta))
                        e.Graphics.FillRectangle(brush, new RectangleF(
                            gradient_rectangle.X + (ticksRectangle.Width * 4 / 6f), gradient_rectangle.Y,
                            gradient_rectangle.Width / 6f, gradient_rectangle.Height));
                    using (Brush brush = new LinearGradientBrush(
                            new PointF(ticksRectangle.X + (ticksRectangle.Width * 5 / 6f), 0), new PointF(ticksRectangle.X + (ticksRectangle.Width * 6 / 6f), 0),
                            Color.Magenta, Color.Red))
                        e.Graphics.FillRectangle(brush, new RectangleF(
                            gradient_rectangle.X + (ticksRectangle.Width * 5 / 6f), gradient_rectangle.Y,
                            gradient_rectangle.Width / 6f, gradient_rectangle.Height));
                    break;
                case Color_Trackbar_Modes.Lightness:
                    using (Brush brush = new LinearGradientBrush(
                            new PointF(ticksRectangle.X - 0.25f, 0), new PointF(ticksRectangle.X + ticksRectangle.Width / 2f - 0.75f, 0),
                            gradient_start_color, gradient_mid_color))
                        e.Graphics.FillRectangle(brush, new RectangleF(
                            gradient_rectangle.X, gradient_rectangle.Y,
                            gradient_rectangle.Width / 2f, gradient_rectangle.Height));
                    using (Brush brush = new LinearGradientBrush(
                            new PointF(ticksRectangle.X + (ticksRectangle.Width / 2f) - 0.25f, 0), new PointF(ticksRectangle.X + ticksRectangle.Width - 0.75f, 0),
                            gradient_mid_color, gradient_end_color))
                        e.Graphics.FillRectangle(brush, new RectangleF(
                            gradient_rectangle.X + (ticksRectangle.Width / 2f), gradient_rectangle.Y,
                            gradient_rectangle.Width / 2f, gradient_rectangle.Height));
                    break;
                default:
                    using (Brush brush = new LinearGradientBrush(new PointF(ticksRectangle.X - 0.25f, 0), new PointF(ticksRectangle.X + ticksRectangle.Width - 0.75f, 0),
                        gradient_start_color, gradient_end_color))
                        e.Graphics.FillRectangle(brush, gradient_rectangle);
                    break;
            }

            TrackBarRenderer.DrawBottomPointingThumb(e.Graphics, thumbRectangle, thumbState);
        }

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            SetupTrackBar();
        }

        // Determine whether the user has clicked the track bar thumb. 
        protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);
            if (!TrackBarRenderer.IsSupported)
                return;

            thumbRectangle.X = CurrentTickXCoordinate();

            if (this.thumbRectangle.Contains(e.Location))
            {
                thumbClicked = true;
                thumbState = TrackBarThumbState.Pressed;
            }

            this.Invalidate();
        }

        // Redraw the track bar thumb if the user has moved it. 
        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);

            if (!TrackBarRenderer.IsSupported)
                return;

            if (thumbClicked == true)
            {
                thumbClicked = false;
                thumbState = TrackBarThumbState.Hot;
                this.Invalidate();
            }
        }

        // Track cursor movements. 
        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);

            if (!TrackBarRenderer.IsSupported)
                return;

            thumbRectangle.X = CurrentTickXCoordinate();

            // The user is moving the thumb. 
            if (thumbClicked == true)
            {
                thumbRectangle.X = CurrentTickXCoordinate();
            }

            // The cursor is passing over the track. 
            else
            {
                thumbState = thumbRectangle.Contains(e.Location) ?
                    TrackBarThumbState.Hot : TrackBarThumbState.Normal;
            }

            Invalidate();
        }
    }
}
