﻿using System;
using System.Collections.Generic;
using System.Linq;
using Nintenlord.ROMHacking.GBA;

namespace GBAFE_Tilesets
{
    static class Terrain_Stats
    {
        readonly static Dictionary<FEVersion, int> TERRAIN_COUNT = new Dictionary<FEVersion, int>
        {
            { FEVersion.FE7, 65 },
        };
        readonly static Dictionary<FEVersion, int> TERRAIN_DATA_POINTER = new Dictionary<FEVersion, int>
        {
            { FEVersion.FE7, 0xBE44F9 },
        };
        #region TERRAIN NAMES
        readonly static Dictionary<FEVersion, string[]> TERRAIN_NAMES = new Dictionary<FEVersion,string[]>
        {
            { FEVersion.FE7, new string[]
                {
                    "Debug",
                    "Plains",
                    "Road",
                    "Village",
                    "Village (Closed)",
                    "House",
                    "Armory",
                    "Vendor",
                    "Arena",
                    "C. Room???",
                    "Fortress",
                    "Gate (Castle)",
                    "Forest",
                    "Thicket",
                    "Sand",
                    "Desert",
                    "River",
                    "Hill",
                    "Peak",
                    "Bridge",
                    "Bridge (Broken)",
                    "Sea",
                    "Lake",
                    "Floor",
                    "Floor (Ward)",
                    "Fence",
                    "Wall",
                    "Wall (Weak)",
                    "Rubble???",
                    "Pillar",
                    "Door",
                    "Throne",
                    "Chest (Opened)",
                    "Chest",
                    "Roof",
                    "Gate (Fort)",
                    "Church???",
                    "Ruins (Destroyed)",
                    "Cliff",
                    "Ballista",
                    "Heavy Ballista",
                    "Killer Ballista",
                    "Flat",
                    "Wreck Tile???",
                    "-- (Castle)",
                    "Stairs",
                    "-- (Village)",
                    "Glacier",
                    "Arena???",
                    "Valley",
                    "Fence???",
                    "Snag",
                    "Bridge (Snag)",
                    "Sky",
                    "Deeps",
                    "Ruins (Visit)",
                    "Inn",
                    "Barrel",
                    "Bone",
                    "Dark",
                    "Water",
                    "Gunnel",
                    "Deck",
                    "Brace",
                    "Mast",
                }
            },
        };
        #endregion
        readonly static Dictionary<FEVersion,Dictionary<string, int>> MOVEMENT_COSTS = new Dictionary<FEVersion,Dictionary<string,int>>
        {
            { FEVersion.FE7, new Dictionary<string, int>
                {
                    { "T2 Infantry", 0xBE3888 },
                    { "T1 Infantry", 0xBE38C9 },
                    { "Armors", 0xBE390A },
                    { "Axers", 0xBE394B },
                    { "Berserker", 0xBE398C },
                    { "Brigand", 0xBE39CD },
                    { "Pirate", 0xBE3A0E },
                    { "Thief", 0xBE3A4F },
                    { "Mages", 0xBE3A90 },
                    { "NPC Infantry", 0xBE3AD1 },
                    { "T1 Cavalry", 0xBE3B12 },
                    { "T2 Cavalry", 0xBE3B53 },
                    { "Nomad", 0xBE3B94 },
                    { "Trooper", 0xBE3BD5 },
                    { "Fliers", 0xBE3C16 },
                    //{ 0xBE3C57, "Fire Dragon, Ice Dragon //Debug
                }
            },
        };
        const int RAIN_OFFSET = 0x451;
        const int SNOW_OFFSET = 0x861;

        public static string parsed_terrain_stats(GBAROM rom, FEVersion version)
        {
            switch (version)
            {
                case FEVersion.FE7:
                    break;
                default:
                    throw new NotImplementedException();
            }

            int terrain_count = TERRAIN_COUNT[version];
            Dictionary<string, byte[,]> movement_costs = MOVEMENT_COSTS[version].ToDictionary(p => p.Key, p => new byte[terrain_count, 3]);
            foreach (var pair in MOVEMENT_COSTS[version])
            {
                byte[] clear_costs = rom.GetData(pair.Value, terrain_count);
                for (int i = 0; i < clear_costs.Length; i++)
                    movement_costs[pair.Key][i, 0] = clear_costs[i];

                byte[] rain_costs = rom.GetData(pair.Value + RAIN_OFFSET, terrain_count);
                for (int i = 0; i < rain_costs.Length; i++)
                    movement_costs[pair.Key][i, 1] = rain_costs[i];

                byte[] snow_costs = rom.GetData(pair.Value + SNOW_OFFSET, terrain_count);
                for (int i = 0; i < snow_costs.Length; i++)
                    movement_costs[pair.Key][i, 2] = snow_costs[i];
            }
            byte[,] terrain_stats = new byte[terrain_count, 6];
            for (int i = 0; i < 6; i++)
            {
                byte[] stats = rom.GetData(TERRAIN_DATA_POINTER[version] + i * terrain_count, terrain_count);
                for (int j = 0; j < terrain_count; j++)
                    terrain_stats[j, i] = stats[j];
            }

            string result = "";
            string headers = "";
            foreach (var pair in MOVEMENT_COSTS)
                headers += pair.Key + "\t";
            result += string.Format("\t\tGnd\tAir\t\t{0}\r\n", headers);
            for (int i = 0; i < TERRAIN_NAMES[version].Length; i++)
                for (int j = 0; j < 3; j++)
                {
                    string weather_costs = "";
                    foreach (var pair in MOVEMENT_COSTS[version])
                        weather_costs += movement_costs[pair.Key][i, j] + "\t";
                    result += string.Format("{0}\t{1}:\t{2}\t{3}\t{4}:\t{5}\r\n",
                        j == 0 ? TERRAIN_NAMES[version][i] : "",
                        j == 0 ? "Avo" : (j == 1 ? "Def" : "Res"),
                        terrain_stats[i, j],
                        terrain_stats[i, j + 3],
                        j == 0 ? "Clear" : (j == 1 ? "Rain" : "Snow"),
                        weather_costs);
                }
            return result;
        }
    }
}
