﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;

namespace GBAFE_Tilesets
{
    class TilesetData
    {
        public TilesetCoreData Data{get; private set; }
        public List<Tuple<byte, Color[]>> Palettes { get; private set; }

        public TilesetData(Tuple<byte, byte, byte> offsets, byte[] graphic, List<Tuple<byte, Color[]>> palettes, byte[] tsa)
        {
            Data = new TilesetCoreData(offsets, graphic, tsa);
            Palettes = palettes;
        }
        public TilesetData(Tuple<byte, byte, byte> offsets, byte[] graphic, IEnumerable<Tuple<byte, byte[]>> raw_palette, byte[] tsa)
            : this(offsets, graphic, raw_palette
                .Select(x => new Tuple<byte, Color[]>(x.Item1, Nintenlord.ROMHacking.GBA.Graphics.GBAPalette.ToPalette(x.Item2, 0, 160)))
                .ToList(), tsa) { }

        public Maybe<byte>[] subtile_palettes()
        {
            Maybe<byte>[] palettes = new Maybe<byte>[Data.Tsa.Length / 9];
            for (int i = 0; i < palettes.Length; i++)
                palettes[i] = new Maybe<byte>();
            for (int i = 0; i < palettes.Length; i++)
            {
                int x = i % 32;
                int y = i / 32;

                for (int j = 0; j < 4; j++)
                {
                    ushort data = (ushort)((Data.Tsa[i * 8 + j * 2 + 1] << 8) + Data.Tsa[i * 8 + j * 2 + 0]);
                    byte palette = (byte)((data & 0xF000) >> 12); // First 4
                    short image_src = (short)(data & 0x03FF); // Last 10
                    if (image_src >= palettes.Length)
                        return null;
                    if (palettes[image_src].IsNothing)
                        palettes[image_src] = palette;
                }
            }
            return palettes;
        }

        internal void remove_palette(int index)
        {
            Palettes.RemoveAt(index);
        }

        internal void copy_palette(int src_index)
        {
            Color[] palette_ary = new Color[Palettes[src_index].Item2.Length];
            Array.Copy(Palettes[src_index].Item2, palette_ary, palette_ary.Length);
            var new_palette = new Tuple<byte, Color[]>(255, palette_ary);
            Palettes.Add(new_palette);
        }

        internal void set_palette(int index, Color[] palette)
        {
            Array.Copy(palette, Palettes[index].Item2, palette.Length);
        }
    }
}
